package com.mtgg.controller;

import com.alibaba.fastjson.JSONObject;
import com.jpay.ext.kit.HttpKit;
import com.jpay.ext.kit.PaymentKit;
import com.jpay.ext.kit.StrKit;
import com.jpay.weixin.api.WxPayApi;
import com.jpay.weixin.api.WxPayApiConfig;
import com.jpay.weixin.api.WxPayApiConfigKit;
import com.mtgg.bean.ConfigBean;
import com.mtgg.bean.PayLogDTO;
import com.mtgg.bean.WXData;
import com.mtgg.common.ErrorCode;
import com.mtgg.common.Response;
import com.mtgg.dto.MtRecordDTO;
import com.mtgg.entity.MtRecordPO;
import com.mtgg.service.BatisService;
import com.mtgg.utils.ExcelUtil;
import com.mtgg.utils.IpUtil;
import com.mtgg.utils.WechatUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.BufferedOutputStream;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * @Author: MTGG 成长，是否是注定失去很多东西，这是一种悲哀，即使你有信心不会失去，但是这个事物就是这样，你站在两座山的中间
 * 你走向一座山，便注定远离另一座山
 * @Date: 19:29 2018/6/25
 * @Describe:
 */
@Slf4j
@Controller
@ResponseBody
@RequestMapping("/wx")
public class WXController {

    @Autowired
    private ConfigBean configBean;
    @Autowired
    private BatisService batisService;


    public WxPayApiConfig getApiConfig() {
        return WxPayApiConfig.New()
                .setAppId(configBean.getWxAppId())
                .setMchId(configBean.getWxMchId())
                .setPaternerKey(configBean.getWxPartnerKey())
                .setPayModel(WxPayApiConfig.PayModel.BUSINESSMODEL);
    }

    /**
     * @Author: MTGG
     * @Date: 19:33 2018/7/4
     * @Describe: 获取openid
     */
    @RequestMapping(value = "/openid", method = RequestMethod.GET)
    public Response getOpenId(@RequestParam("code") String code) {
        Response jr = new Response();
        log.info("接收到的code：{}", code);
        //根据code获取openid
        try {
            JSONObject jsonObject = WechatUtils.getInstance().getDriverAuthAccessToken(code);
            log.info("获取到的openid等信息：{}", jsonObject);
            WXData wxData = new WXData();
            wxData.setOpenid(jsonObject.getString("openid"));
            wxData.setSessionKey(jsonObject.getString("sessionKey"));
            jr.setObj(wxData);
        } catch (Exception e) {
            log.info("出错：{}", e);
        }
        jr.setRes(ErrorCode.SUCCESS.code);
        return jr;
    }

    /**
     * @Author: MTGG
     * @Date: 19:36 2018/6/25
     * @Describe: 公众号支付封装
     */
    @RequestMapping(value = "/gzhPay", method = RequestMethod.POST)
    public Response webPay(@RequestParam("totalFee") String totalFee,
                           @RequestParam("account") String account,
                           @RequestParam("openid") String openid,
                           HttpServletRequest request) {
        Response jr = new Response();
        log.info("openid:{};金额：{}；账号：{}", openid, totalFee, account);
        String total_fee = totalFee.concat("00");
        log.info("total_fee:{}", total_fee);

        if (StrKit.isBlank(openid)) {
            jr.setRes(ErrorCode.ERR_NOT_EXIST.code);
            jr.setMsg("获取到的openid为空");
            return jr;
        }
        String ip = IpUtil.getIpAddr(request);
        if (StrKit.isBlank(ip)) {
            ip = "127.0.0.1";
        }
        WxPayApiConfigKit.putApiConfig(getApiConfig());
        log.info("获取到的ip：{}", ip);
        Map<String, String> params = WxPayApiConfigKit.getWxPayApiConfig()
                .setAttach(account)
                .setBody("佳禾顺")
                .setOpenId(openid)
                .setSpbillCreateIp(ip)
                .setTotalFee(total_fee)
                .setTradeType(WxPayApi.TradeType.JSAPI)
                .setNotifyUrl(configBean.getWxNotifyUrl())
                .setOutTradeNo(String.valueOf(System.currentTimeMillis()))
                .build();
        String xmlResult = WxPayApi.pushOrder(false, params);
        Map<String, String> resultMap = PaymentKit.xmlToMap(xmlResult);
        log.info("封装后解析后的xml：{}", resultMap);
        String return_code = resultMap.get("return_code");
        if (!PaymentKit.codeIsOK(return_code)) {
            jr.setRes(ErrorCode.FAIL.code);
            jr.setMsg("return_code is not ok");
            return jr;
        }
        String result_code = resultMap.get("result_code");
        if (!PaymentKit.codeIsOK(result_code)) {
            jr.setRes(ErrorCode.FAIL.code);
            jr.setMsg("result_code is not ok");
            return jr;
        }

        // 以下字段在return_code 和result_code都为SUCCESS的时候有返回
        String prepay_id = resultMap.get("prepay_id");
        String timeStamp = String.valueOf(System.currentTimeMillis() / 1000);
        String nonce_str = String.valueOf(System.currentTimeMillis());
        String packages = "prepay_id=" + prepay_id;
        Map<String, String> packageParams = new HashMap<String, String>();
        packageParams.put("appId", configBean.getWxAppId());
        packageParams.put("timeStamp", timeStamp);
        packageParams.put("nonceStr", nonce_str);
        packageParams.put("package", "prepay_id=" + prepay_id);
        packageParams.put("signType", "MD5");
        String packageSign = PaymentKit.createSign(packageParams, WxPayApiConfigKit.getWxPayApiConfig().getPaternerKey());
        packageParams.put("paySign", packageSign);
        Map<String, Object> map = new HashMap<String, Object>();
        map.put("appid", configBean.getWxAppId());
        map.put("timeStamp", timeStamp);
        map.put("nonceStr", nonce_str);
        map.put("packageValue", packages);
        map.put("paySign", packageSign);
        map.put("success", "ok");
        jr.setRes(ErrorCode.SUCCESS.code);
        jr.setObj(map);
        return jr;
    }

    /**
     * @Author: MTGG
     * @Date: 21:10 2018/7/8
     * @Describe: 支付回调
     */
    @RequestMapping(value = "/payNotify", method = {RequestMethod.POST, RequestMethod.GET})
    public void pay_notify(HttpServletRequest request, HttpServletResponse response) {
        // 支付结果通用通知文档: https://pay.weixin.qq.com/wiki/doc/api/jsapi.php?chapter=9_7
        log.info("---------------支付回调----------------");
        String xmlMsg = HttpKit.readData(request);
        Map<String, String> params = PaymentKit.xmlToMap(xmlMsg);
        String resXml = "";
        String result_code = params.get("result_code");
        WxPayApiConfigKit.setThreadLocalWxPayApiConfig(getApiConfig());
        if (PaymentKit.verifyNotify(params, WxPayApiConfigKit.getWxPayApiConfig().getPaternerKey())) {
            if (("SUCCESS").equals(result_code)) {
                log.info("验证成功，下面处理业务");
                // 微信支付订单号
                String transaction_id = params.get("transaction_id");
                log.info("微信支付订单号={}", transaction_id);
                // 商户订单号
                String out_trade_no = params.get("out_trade_no");
                log.info("商户订单号={}", out_trade_no);
                String total_fee = params.get("total_fee");
                total_fee = total_fee.substring(0, total_fee.length() - 2);
                String account = params.get("attach");
                log.info("账户：{}；金额：{}", account, total_fee);

                MtRecordPO mtRecordPO = new MtRecordPO();
                mtRecordPO.setUsername(account);
                mtRecordPO.setMoney(Integer.parseInt(total_fee));
                mtRecordPO.setOrderSn(out_trade_no);
                mtRecordPO.setPayOrderSn(transaction_id);
                mtRecordPO.setStatus((byte) 0);
                mtRecordPO.setAddTime(new Date());
                log.info("下面记录insert");
                int ret = batisService.insertSelective(mtRecordPO);
                log.info("添加记录返回：{}", ret);
                resXml = "<xml>" + "<return_code><![CDATA[SUCCESS]]></return_code>"
                        + "<return_msg><![CDATA[OK]]></return_msg>" + "</xml> ";
            } else {
                //TODO FAIL支付失败
                log.debug("支付失败的回调消息");
                resXml = "<xml>" + "<return_code><![CDATA[FAIL]]></return_code>"
                        + "<return_msg><![CDATA[报文为空]]></return_msg>" + "</xml> ";
            }
            BufferedOutputStream out = null;
            try {
                log.info("闭嘴");
                out = new BufferedOutputStream(
                        response.getOutputStream());
                out.write(resXml.getBytes());
                out.flush();
                out.close();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        log.info("验证失败");
    }
}
