package com.mtgg.utils;

import com.alibaba.fastjson.JSONObject;
import com.mtgg.bean.ConfigBean;
import com.mtgg.common.Constant;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.HashMap;
import java.util.Map;

/**
 * 微信开发组件
 * 单例, 线程安全
 *
 * @author bostin.wang
 * @date 2017-12-26
 */
@Slf4j
public class WechatUtils {

    @Autowired
    private ConfigBean configBean;

    public String getDriverBaseAccessToken() {
        return getBaseAccessToken(appId, secret);
    }

    public JSONObject getDriverAuthAccessToken(String code) {
        return getAuthAccessToken(code, appId, secret);
    }

    /**
     * 获取jsapi调用的tickt
     *
     * @param accessToken
     * @return
     */
    public String getJsapiTicket(String accessToken) {
        if (StringUtils.isBlank(accessToken)) {
            return null;
        }
        String reqUrl = "https://api.weixin.qq.com/cgi-bin/ticket/getticket";
        Map<String, String> params = new HashMap<String, String>();
        params.put("access_token", accessToken);
        params.put("type", "jsapi");
        try {
//            String jsonStr = HttpUtils.sendGet(reqUrl, params);
//            JSONObject data = JSONObject.parseObject(jsonStr);
//            if (data != null) {
//                if (data.getInteger("errcode") == 0) {
//                    return data.getString("ticket");
//                } else if (data.getInteger("errcode").equals(Constant.ACCESS_TOKEN_ERR)) {
//                    System.out.println("access_token过期{}");
//                    //使用这个返回值通知调用方token失效了
//                    return Constant.ACCESS_TOKEN_INVALID;
//                } else {
//                    System.out.println("获取ticket异常");
//                }
//            }
        } catch (Exception e) {
            System.out.println("获取ticket异常");
        }
        return null;
    }

    /**
     * 获取微信用户信息
     *
     * @param access_token
     * @param openid
     * @return
     */
    public JSONObject getWXUserInfo(String access_token, String openid) {
        String reqUrl = "https://api.weixin.qq.com/sns/userinfo";
        Map<String, String> params = new HashMap<String, String>();
        params.put("access_token", access_token);
        params.put("openid", openid);
        params.put("lang", "zh_CN");
        try {
//            String jsonStr = HttpUtils.sendGet(reqUrl, params);
//            JSONObject data = JSONObject.parseObject(jsonStr);
//            if (data != null) {
//                if (data.containsKey("errcode")) {
//                    System.out.println("获取userinfo异常");
//                } else {
//                    return data;
//                }
//            }
        } catch (Exception e) {
            System.out.println("获取snsapi_base异常");
        }
        return null;
    }

    /**
     * 微信公众号基础access token
     *
     * @param appId
     * @param appSecret
     * @return
     */
    private String getBaseAccessToken(String appId, String appSecret) {
        String reqUrl = "https://api.weixin.qq.com/cgi-bin/token";
        Map<String, String> params = new HashMap<String, String>();
        params.put("grant_type", "client_credential");
        params.put("appid", appId);
        params.put("secret", appSecret);
        try {
//            String jsonStr = HttpUtils.sendGet(reqUrl, params);
//            JSONObject data = JSONObject.parseObject(jsonStr);
//            if (data != null) {
//                if (data.containsKey("errcode")) {
//                    System.out.println("accesstoken");
//                } else {
//                    return data.getString("access_token");
//                }
//            }
        } catch (Exception e) {
            System.out.println("accesstoken");
        }
        return null;
    }

    /**
     * 根据微信网页授权code，获取openid，access_token（网页授权接口调用凭证）等
     *
     * @param code
     * @return
     */
    private JSONObject getAuthAccessToken(String code, String appId, String appSecret) {
        String reqUrl = "https://api.weixin.qq.com/sns/oauth2/access_token";
        Map<String, String> params = new HashMap<String, String>();
        params.put("code", code);
        params.put("appid", appId);
        params.put("secret", appSecret);
        params.put("grant_type", "authorization_code");
        try {
//            String jsonStr = HttpUtils.sendGet(reqUrl, params);
//            JSONObject data = JSONObject.parseObject(jsonStr);
//            log.info("获取openid信息返回：{}",data);
//            if (data != null) {
//                if (data.containsKey("errcode")) {
//                    log.info("获取snsapi_base异常");
//                } else {
//                    return data;
//                }
//            }
        } catch (Exception e) {
            log.info("获取snsapi_base异常");
        }
        return null;
    }


    private WechatUtils() {
        appId = configBean.getWxAppId();
        secret = configBean.getWxSecret();
    }

    private static class SingletonInstance {
        private static final WechatUtils INSTANCE = new WechatUtils();
    }

    public static WechatUtils getInstance() {
        return SingletonInstance.INSTANCE;
    }

    private String appId;
    private String secret;

}